@@ -12,6 +12,34 @@ def cppa(el):
     return '{{{}}}{}'.format(_NSMAP['cppa'], el)
 
 cppa3_content_model = {
+
+    cppa('AS1Channel') : [cppa('Description'),
+                          cppa('ChannelProfile'),
+                          cppa('MaxSize'),
+                          cppa('Signature'),
+                          cppa('Encryption'),
+                          cppa('ErrorHandling'),
+                          cppa('ReceiptHandling'),
+                          cppa('Compression')],
+
+    cppa('AS2Channel') : [cppa('Description'),
+                          cppa('ChannelProfile'),
+                          cppa('MaxSize'),
+                          cppa('Signature'),
+                          cppa('Encryption'),
+                          cppa('ErrorHandling'),
+                          cppa('ReceiptHandling'),
+                          cppa('Compression')],
+
+    cppa('AS3Channel') : [cppa('Description'),
+                          cppa('ChannelProfile'),
+                          cppa('MaxSize'),
+                          cppa('Signature'),
+                          cppa('Encryption'),
+                          cppa('ErrorHandling'),
+                          cppa('ReceiptHandling'),
+                          cppa('Compression')],
+
     cppa('ebMS2Channel') : [cppa('Description'),
                             cppa('ChannelProfile'),
                             cppa('MaxSize'),
