@@ -30,7 +30,5 @@ class CPPA2TestCase( unittest.TestCase ):
         fd.close()
 
 
-    def test_4000(self):
-        self.do_downtranslate('4000')
 
 
