@@ -51,7 +51,8 @@ class CPPA3TestCase( unittest.TestCase ):
                        acpp_url=None,
                        bcpp_url=None,
                        handle_defaults=False,
-                       remove_unused_certs=True):
+                       remove_unused_certs=True,
+                       agreementidfun=None):
         logging.info('------------------------------------------')
         logging.info('Running test {}'.format(id))
         #logging.basicConfig(level=logging.DEBUG,
@@ -85,7 +86,8 @@ class CPPA3TestCase( unittest.TestCase ):
                     default_handler = self.channel_profile_handler,
                     handle_defaults=handle_defaults,
                     delegation_handler=is_connected_to,
-                    remove_unused_certs=remove_unused_certs
+                    remove_unused_certs=remove_unused_certs,
+                    agreementidfun=agreementidfun
         )
         if self.schema is not None:
             # Change the serial number as many CAs generate values that are
@@ -492,6 +494,37 @@ class CPPA3TestCase( unittest.TestCase ):
     @@@ to do
     """
 
+    # One Service with complementary Roles one matching Action, matching send-receive, success
+    # Consistent ebBP uuid
+    def test_0600(self):
+        self.do_unification('0600')
+
+    # One Service with complementary Roles one matching Action, matching send-receive, success
+    # Inconsistent ebBP uuid
+    def test_0601(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '0601')
+
+    # One Service with complementary Roles one matching Action, matching send-receive, success
+    # Consistent ebBP uuid and name
+    def test_0602(self):
+        self.do_unification('0602')
+
+    # One Service with complementary Roles one matching Action, matching send-receive, success
+    # Consistent ebBP uuid but inconsitent name
+    def test_0603(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '0603')
+
+
+    # Both A and B support version "1.0" and "2.0" of a process.  This matches correctly
+    def test_0610(self):
+        self.do_unification('0610')
+
+
+    # A support version "1.0" and "2.0" of a process, B only "1.0".  Only "1.0" in result
+    def test_0611(self):
+        self.do_unification('0611')
+
+
 
     """
     Access Control feature
@@ -958,6 +991,18 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_1611(self):
         self.do_unification('1611', handle_defaults=True)
 
+    # Test 1604,  updated for ENTSOG profile v3,  no default processing
+    def test_1620(self):
+        self.do_unification('1620', handle_defaults=False, agreementidfun=entsog_agreementref)
+
+    # Test 1604,  updated for ENTSOG profile v3,  but referenced certs, no default processing
+    def test_1621(self):
+        self.do_unification('1621', handle_defaults=False, agreementidfun=entsog_agreementref)
+
+    # Test 1621,  updated for ENTSOG profile v3,  but referenced certs, default processing
+    def test_1622(self):
+        self.do_unification('1622', handle_defaults=True, agreementidfun=entsog_agreementref)
+
     # SuperAnnuation
     # As defined in DATA AND PAYMENT STANDARDS
     # MESSAGE ORCHESTRATION AND PROFILES
@@ -1254,6 +1299,22 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_3050(self):
         self.do_unification('3050')
 
+    # 1621,  adapted for AS2 for EASEE-gas portal developers
+    def test_3060(self):
+        self.do_unification('3060', handle_defaults=False)
+
+    # 1621,  adapted for AS2 for EASEE-gas portal developers with default handling
+    def test_3061(self):
+        self.do_unification('3061', handle_defaults=True)
+
+    # Wildcard service, action and role
+    def test_3062(self):
+        self.do_unification('3062', handle_defaults=False)
+
+    # Wildcard service, action and role
+    def test_3063(self):
+        self.do_unification('3063', handle_defaults=True)
+
     # AS1 example
     def test_3100(self):
         self.do_unification('3100')
@@ -1408,6 +1469,10 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_5005(self):
         self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '5005')
 
+    # Like 5002,  but also a receiving channel
+    def test_5006(self):
+        self.do_unification('5006')
+
     # Version of test 3400 using a delegation channel
     def test_5100(self):
         self.do_unification('5100')
@@ -1721,3 +1786,11 @@ def is_connected_to(
         return True
     else:
         return False
+
+def entsog_agreementref(acpp, bcpp, version=1):
+    a_party_id = acpp.xpath('child::cppa:PartyInfo/cppa:PartyId[1]/text()',
+                            namespaces=NSMAP)[0]
+    b_party_id = bcpp.xpath('child::cppa:PartyInfo/cppa:PartyId[1]/text()',
+                            namespaces=NSMAP)[0]
+    [first, second] = sorted([a_party_id, b_party_id])
+    return "http://entsog.eu/communication/agreements/{}/{}/{}".format(first, second, version)
